# Copyright 2009-2011 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
Resources for `test_import_tools.test_exists.test_zip`.

Contains an archive with a Python module inside, which `exists` should be able
to locate.
'''
