# Copyright 2009-2011 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''Package for caching of functions that take history browsers.'''
# todo: reorganize?

from .decorators import history_cache