'''
simpack_name description.
'''
# (The above docstring should be in ReStructuredText.)

from .state import State

# Human-readable name of your simpack:
name = '''simpack_name'''

# List of tags like 'game-theory', 'electronics', 'climate', etc.
tags = []