# Copyright 2009-2011 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
Defines `profile_ready`, a decorator for flexibly profiling a function..

See its documentation for more details.
'''

from . import base_profile
from .cute_profile import profile_ready