# Copyright 2009-2011 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''Defines tools for tracing function calls.'''

from .count_calls import count_calls
from .temp_function_call_counter import TempFunctionCallCounter